import React, { PropTypes } from 'react'
import { render } from 'react-dom'
import { createStore, combineReducers } from 'redux'
import { Provider, connect } from 'react-redux'
import { markdown } from 'markdown'

const UPDATE_MARKDOWN = 'UPDATE_MARKDOWN'

// Action Creator
function updateMarkdown(markdown) {
  return { 
    type: UPDATE_MARKDOWN, 
    markdown 
  }
}

// Reducer
function mdtext(state = '', action) {
  switch(action.type) {
  case UPDATE_MARKDOWN:
    return action.markdown
  default:
    return state
  }
}

const markdownApp = combineReducers({ mdtext })
let store = createStore(markdownApp, { mdtext: '' })

store.subscribe(() => console.log(store.getState()))
store.dispatch(updateMarkdown('# Hello, World!!!!!'))

// Presentational Components
const App = ({html}) => (
  <div>
    <MarkdownForm />
    <p dangerouslySetInnerHTML={{__html: html}}></p>
  </div>
)

App.propTypes = {
  html: PropTypes.string.isRequired
}

// Container Components
function getMarkdownHTML(mdtext) {
  return markdown.toHTML(mdtext)
}

const mapStateToProps = (state) => {
  return {html: getMarkdownHTML(state.mdtext)}
}

const MarkdownApp = connect(mapStateToProps)(App)

let MarkdownForm = ({dispatch}) => {
  return (
    <textarea onChange={e => {
        dispatch(updateMarkdown(e.target.value))
      }}></textarea>
  )
}
MarkdownForm = connect()(MarkdownForm)

render(
  <Provider store={store}>
    <MarkdownApp />
  </Provider>,
  document.getElementById('container')
)
